@@ -1,8 +1,79 @@
-#![cfg_attr(docsrs, doc = include_str!("../README.md"))]
+#![cfg_attr(docsrs, doc = include_str!("../docs/index.md"))]
 
 pub use include_sql::include_sql;
 
-/// Generates Rust code to use included SQL.
+/**
+Generates Rust code to use included SQL.
+
+This macro defines a trait with methods to access data and implements it for `postgres::Client`.
+
+This macro recognizes and generates 3 variants of database access methods using the following selectors:
+* `?` - methods that process rows retrieved by `SELECT`,
+* `!` - methods that execute all other non-`SELECT` methods, and
+* `->` - methods that execute `RETURNING` statements and provide access to returned data.
+
+For `SELECT` statements (`?`) like:
+
+```sql
+-- name: get_loaned_books?
+-- param: user_id: &str
+SELECT book_title FROM library WHERE loaned_to = :user_id;
+```
+
+The method with the following signature is generated:
+
+```rust , ignore
+fn get_loaned_books<F>(
+    &self,
+    user_id: &str,
+    row_callback: F
+) -> Result<(),postgres::Error>
+where F: Fn(postgres::Row) -> Result<(),postgres::Error>;
+```
+
+For non-select statements (`!`) - INSERT, UPDATE, DELETE, etc. - like:
+
+```sql
+-- name: loan_books!
+-- param: user_id: &str
+-- param: book_ids: i32
+UPDATE library
+   SET loaned_to = :user_id
+     , loaned_on = current_timestamp
+ WHERE book_id IN (:book_ids);
+```
+
+The method with the following signature is generated:
+
+```rust , ignore
+fn loan_books(
+    &self,
+    user_id: &str,
+    book_ids: &[i32]
+) -> Result<u64,postgres::Error>;
+```
+
+For DELETE, INSERT, and UPDATE statements that return data via `RETURNING` clause (`->`) like:
+
+```sql
+-- name: add_new_book->
+-- param: isbn: &str
+-- param: book_title: &str
+INSERT INTO library (isbn, book_title)
+VALUES (:isbn, :book_title)
+RETURNING book_id;
+```
+
+The method with the following signature is generated:
+
+```rust , ignore
+fn add_new_book(
+    &self,
+    isbn: &str,
+    book_title: &str
+) -> Result<postgres::Row,postgres::Error>;
+```
+*/
 #[macro_export]
 macro_rules! impl_sql {
     ( $sql_name:ident = $( { $kind:tt $name:ident ($($variant:tt $param:ident $ptype:tt)*) $doc:literal $s:tt $( $text:tt )+ } ),+ ) => {
@@ -19,6 +90,7 @@ macro_rules! impl_sql {
 }
 
 #[macro_export]
+#[doc(hidden)]
 macro_rules! decl_method {
     ( ? $name:ident $doc:literal ($($gen_type:ident)*) ($($fn_params:tt)*) ) => {
         #[doc=$doc]
@@ -76,6 +148,7 @@ macro_rules! decl_method {
 }
 
 #[macro_export]
+#[doc(hidden)]
 macro_rules! impl_method {
     ( ? $name:ident () () () => () $text:literal ) => {
         fn $name<F>(&mut self, row_cb: F) -> std::result::Result<(),postgres::Error>
@@ -218,6 +291,7 @@ macro_rules! impl_method {
 }
 
 #[macro_export]
+#[doc(hidden)]
 macro_rules! sql_literal {
     ($($name:ident)+ => $text:literal) => {
         $text
@@ -234,6 +308,7 @@ macro_rules! sql_literal {
 }
 
 #[macro_export]
+#[doc(hidden)]
 macro_rules! num_args {
     () => { 0 };
     (: $head:ident $($tail:tt)*) => { 1 + $crate::num_args!($($tail)*) };
@@ -241,6 +316,7 @@ macro_rules! num_args {
 }
 
 #[macro_export]
+#[doc(hidden)]
 macro_rules! sql_len {
     () => { 0 };
     ($text:literal $($tail:tt)*) => { $text.len() + $crate::sql_len!($($tail)*) };
@@ -249,6 +325,7 @@ macro_rules! sql_len {
 }
 
 #[macro_export]
+#[doc(hidden)]
 macro_rules! dynamic_sql {
     ($stmt:ident $args:ident $i:ident) => {};
     ($stmt:ident $args:ident $i:ident $text:literal $($tail:tt)*) => {
